@@ -1,21 +1,27 @@
 //! Code to represent its single guard node and track its status.
 
+use tor_basic_utils::retry::RetryDelay;
 use tor_linkspec::ChanTarget;
 use tor_llcrypto::pk::{ed25519::Ed25519Identity, rsa::RsaIdentity};
 use tor_netdir::{NetDir, Relay, RelayWeight};
 
+use educe::Educe;
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::net::SocketAddr;
 use std::time::{Duration, Instant, SystemTime};
 use tracing::{trace, warn};
 
+use crate::dirstatus::DirStatus;
+use crate::skew::SkewObservation;
 use crate::util::randomize_time;
-use crate::{GuardId, GuardParams, GuardRestriction, GuardUsage};
+use crate::{ids::GuardId, GuardParams, GuardRestriction, GuardUsage};
+use crate::{ExternalActivity, FirstHopId, GuardUsageKind};
 use tor_persist::{Futureproof, JsonValue};
 
 /// Tri-state to represent whether a guard is believed to be reachable or not.
-#[derive(Debug, Clone, Copy, Eq, PartialEq)]
+#[derive(Debug, Clone, Copy, Eq, PartialEq, Educe)]
+#[educe(Default)]
 #[allow(clippy::enum_variant_names)]
 pub(crate) enum Reachable {
     /// A guard is believed to be reachable, since we have successfully
@@ -30,15 +36,10 @@ pub(crate) enum Reachable {
     ///   * We haven't tried to use the guard.
     ///   * Attempts to use it have failed, but those attempts are far
     ///     enough in the past that we're willing to retry them.
+    #[educe(Default)]
     Unknown,
 }
 
-impl Default for Reachable {
-    fn default() -> Self {
-        Reachable::Unknown
-    }
-}
-
 /// The name and version of the crate that first picked a potential
 /// guard.
 ///
@@ -131,23 +132,33 @@ pub(crate) struct Guard {
     /// retry it?
     ///
     /// (Retrying a guard involves clearing this field, and setting
-    /// `reachable`
+    /// `reachable`)
     #[serde(skip)]
-    retry_at: Option<Instant>, // derived from tried_to_connect_at.
+    retry_at: Option<Instant>, // derived from retry_schedule.
+
+    /// Schedule use to determine when we can next attempt to connect to this
+    /// guard.
+    #[serde(skip)]
+    retry_schedule: Option<RetryDelay>,
 
     /// Current reachability status for this guard.
     #[serde(skip)]
     reachable: Reachable,
 
-    /// If this guard is currently failing, when did it start to fail?
-    #[serde(skip)]
-    failing_since: Option<Instant>,
-
     /// If true, then the last time we saw a relay entry for this
     /// guard, it seemed like a valid directory cache.
     #[serde(skip)]
     is_dir_cache: bool,
 
+    /// Status for this guard, when used as a directory cache.
+    ///
+    /// (This is separate from `Reachable` and `retry_schedule`, since being
+    /// usable for circuit construction does not necessarily mean that the guard
+    /// will have good, timely cache information.  If it were not separate, then
+    /// circuit success would clear directory failures.)
+    #[serde(skip, default = "guard_dirstatus")]
+    dir_status: DirStatus,
+
     /// If true, we have given this guard out for an exploratory circuit,
     /// and that exploratory circuit is still pending.
     ///
@@ -167,12 +178,25 @@ pub(crate) struct Guard {
     #[serde(skip)]
     suspicious_behavior_warned: bool,
 
+    /// Latest clock skew (if any) we have observed from this guard.
+    #[serde(skip)]
+    clock_skew: Option<SkewObservation>,
+
     /// Fields from the state file that was used to make this `Guard` that
     /// this version of Arti doesn't understand.
     #[serde(flatten)]
     unknown_fields: HashMap<String, JsonValue>,
 }
 
+/// Lower bound for delay after get a failure using a guard as a directory
+/// cache.
+const GUARD_DIR_RETRY_FLOOR: Duration = Duration::from_secs(60);
+
+/// Return a DirStatus entry for a guard.
+fn guard_dirstatus() -> DirStatus {
+    DirStatus::new(GUARD_DIR_RETRY_FLOOR)
+}
+
 /// Wrapper to declare whether a given successful use of a guard is the
 /// _first_ successful use of the guard.
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
@@ -195,7 +219,11 @@ impl Guard {
             params.lifetime_unconfirmed / 10,
         );
 
-        Self::new(GuardId::from_relay(relay), relay.addrs().into(), added_at)
+        Self::new(
+            GuardId::from_chan_target(relay),
+            relay.addrs().into(),
+            added_at,
+        )
     }
 
     /// Return a new, manually constructed [`Guard`].
@@ -206,18 +234,19 @@ impl Guard {
             added_at,
             added_by: CrateId::this_crate(),
             disabled: None,
-
             confirmed_at: None,
             unlisted_since: None,
             microdescriptor_missing: false,
             last_tried_to_connect_at: None,
             reachable: Reachable::Unknown,
-            failing_since: None,
             retry_at: None,
+            dir_status: guard_dirstatus(),
+            retry_schedule: None,
             is_dir_cache: true,
             exploratory_circ_pending: false,
             circ_history: CircHistory::default(),
             suspicious_behavior_warned: false,
+            clock_skew: None,
             unknown_fields: Default::default(),
         }
     }
@@ -232,24 +261,76 @@ impl Guard {
         self.reachable
     }
 
+    /// Return the next time at which this guard will be retriable for a given
+    /// usage.
+    ///
+    /// (Return None if we think this guard might be reachable right now.)
+    pub(crate) fn next_retry(&self, usage: &GuardUsage) -> Option<Instant> {
+        match &usage.kind {
+            GuardUsageKind::Data => self.retry_at,
+            GuardUsageKind::OneHopDirectory => [self.retry_at, self.dir_status.next_retriable()]
+                .iter()
+                .flatten()
+                .max()
+                .copied(),
+        }
+    }
+
     /// Return true if this guard is listed in the latest NetDir, and hasn't
     /// been turned off for some other reason.
     pub(crate) fn usable(&self) -> bool {
         self.unlisted_since.is_none() && self.disabled.is_none()
     }
 
+    /// Return true if this guard is ready (with respect to any timeouts) for
+    /// the given `usage` at `now`.
+    pub(crate) fn ready_for_usage(&self, usage: &GuardUsage, now: Instant) -> bool {
+        if let Some(retry_at) = self.retry_at {
+            if retry_at > now {
+                return false;
+            }
+        }
+
+        match usage.kind {
+            GuardUsageKind::Data => true,
+            GuardUsageKind::OneHopDirectory => self.dir_status.usable_at(now),
+        }
+    }
+
     /// Copy all _non-persistent_ status from `other` to self.
     ///
     /// Requires that the two `Guard`s have the same ID.
-    pub(crate) fn copy_status_from(&mut self, other: &Guard) {
+    pub(crate) fn copy_status_from(self, other: Guard) -> Guard {
         debug_assert_eq!(self.id, other.id);
 
-        self.last_tried_to_connect_at = other.last_tried_to_connect_at;
-        self.retry_at = other.retry_at;
-        self.reachable = other.reachable;
-        self.failing_since = other.failing_since;
-        self.is_dir_cache = other.is_dir_cache;
-        self.exploratory_circ_pending = other.exploratory_circ_pending;
+        Guard {
+            // All persistent fields are taken from `self`.
+            id: self.id,
+            orports: self.orports,
+            added_at: self.added_at,
+            added_by: self.added_by,
+            disabled: self.disabled,
+            confirmed_at: self.confirmed_at,
+            unlisted_since: self.unlisted_since,
+            unknown_fields: self.unknown_fields,
+
+            // All non-persistent fields get taken from `other`.
+            last_tried_to_connect_at: other.last_tried_to_connect_at,
+            retry_at: other.retry_at,
+            retry_schedule: other.retry_schedule,
+            reachable: other.reachable,
+            is_dir_cache: other.is_dir_cache,
+            exploratory_circ_pending: other.exploratory_circ_pending,
+            microdescriptor_missing: other.microdescriptor_missing,
+            circ_history: other.circ_history,
+            suspicious_behavior_warned: other.suspicious_behavior_warned,
+            dir_status: other.dir_status,
+            clock_skew: other.clock_skew,
+            // Note that we _could_ remove either of the above blocks and add
+            // `..self` or `..other`, but that would be risky: it would increase
+            // the odds that we would forget to add some persistent or
+            // non-persistent field to the right group in the future.
+        }
     }
 
     /// Change the reachability status for this guard.
@@ -300,6 +381,7 @@ impl Guard {
         if self.reachable != Reachable::Reachable {
             self.set_reachable(Reachable::Unknown);
             self.retry_at = None;
+            self.retry_schedule = None;
         }
     }
 
@@ -311,14 +393,13 @@ impl Guard {
     /// Return true if this guard obeys a single restriction.
     fn obeys_restriction(&self, r: &GuardRestriction) -> bool {
         match r {
-            GuardRestriction::AvoidId(ed) => &self.id.ed25519 != ed,
-            GuardRestriction::AvoidAllIds(ids) => !ids.contains(&self.id.ed25519),
+            GuardRestriction::AvoidId(ed) => &self.id.0.ed25519 != ed,
+            GuardRestriction::AvoidAllIds(ids) => !ids.contains(&self.id.0.ed25519),
         }
     }
 
     /// Return true if this guard is suitable to use for the provided `usage`.
     pub(crate) fn conforms_to_usage(&self, usage: &GuardUsage) -> bool {
-        use crate::GuardUsageKind;
         match usage.kind {
             GuardUsageKind::OneHopDirectory => {
                 if !self.is_dir_cache {
@@ -343,7 +424,7 @@ impl Guard {
     /// download another microdescriptor before we can be certain whether this
     /// guard is listed or not.
     pub(crate) fn listed_in(&self, netdir: &NetDir) -> Option<bool> {
-        netdir.id_pair_listed(&self.id.ed25519, &self.id.rsa)
+        netdir.id_pair_listed(&self.id.0.ed25519, &self.id.0.rsa)
     }
 
     /// Change this guard's status based on a newly received or newly
@@ -360,11 +441,9 @@ impl Guard {
         // not.
         let listed_as_guard = match self.listed_in(netdir) {
             Some(true) => {
+                let id: FirstHopId = self.id.clone().into();
                 // Definitely listed.
-                let relay = self
-                    .id
-                    .get_relay(netdir)
-                    .expect("Couldn't get a listed relay?!");
+                let relay = id.get_relay(netdir).expect("Couldn't get a listed relay?!");
                 // Update address information.
                 self.orports = relay.addrs().into();
                 // Check whether we can currently use it as a directory cache.
@@ -453,18 +532,17 @@ impl Guard {
     ///
     /// If `is_primary` is true, this is a primary guard (q.v.).
     pub(crate) fn record_failure(&mut self, now: Instant, is_primary: bool) {
-        let failing_since = self.failing_since.get_or_insert(now);
-        let failing_time = now.saturating_duration_since(*failing_since);
         self.set_reachable(Reachable::Unreachable);
         self.exploratory_circ_pending = false;
 
-        let connect_attempt = self.last_tried_to_connect_at.unwrap_or(now);
+        let mut rng = rand::thread_rng();
+        let retry_interval = self
+            .retry_schedule
+            .get_or_insert_with(|| retry_schedule(is_primary))
+            .next_delay(&mut rng);
 
-        // This matches tor, but not the spec.
-        let retry_interval = retry_interval(is_primary, failing_time);
-
-        // TODO-SPEC: Oughtn't we randomize this?
-        self.retry_at = Some(connect_attempt + retry_interval);
+        // TODO-SPEC: Document this behavior in guard-spec.
+        self.retry_at = Some(now + retry_interval);
 
         self.circ_history.n_failures += 1;
     }
@@ -502,8 +580,8 @@ impl Guard {
         now: SystemTime,
         params: &GuardParams,
     ) -> NewlyConfirmed {
-        self.failing_since = None;
         self.retry_at = None;
+        self.retry_schedule = None;
         self.set_reachable(Reachable::Reachable);
         self.exploratory_circ_pending = false;
         self.circ_history.n_successes += 1;
@@ -526,6 +604,24 @@ impl Guard {
         }
     }
 
+    /// Record that an external operation has succeeded on this guard.
+    pub(crate) fn record_external_success(&mut self, how: ExternalActivity) {
+        match how {
+            ExternalActivity::DirCache => {
+                self.dir_status.note_success();
+            }
+        }
+    }
+
+    /// Record that an external operation has failed on this guard.
+    pub(crate) fn record_external_failure(&mut self, how: ExternalActivity, now: Instant) {
+        match how {
+            ExternalActivity::DirCache => {
+                self.dir_status.note_failure(now);
+            }
+        }
+    }
+
     /// Note that a circuit through this guard died in a way that we couldn't
     /// necessarily attribute to the guard.
     pub(crate) fn record_indeterminate_result(&mut self) {
@@ -561,16 +657,33 @@ impl Guard {
     /// We use this information to decide whether we are about to sample
     /// too much of the network as guards.
     pub(crate) fn get_weight(&self, dir: &NetDir) -> Option<RelayWeight> {
-        dir.weight_by_rsa_id(&self.id.rsa, tor_netdir::WeightRole::Guard)
+        dir.weight_by_rsa_id(&self.id.0.rsa, tor_netdir::WeightRole::Guard)
     }
 
-    /// Return a [`crate::Guard`] object to represent this guard.
-    pub(crate) fn get_external_rep(&self) -> crate::Guard {
-        crate::Guard {
-            id: self.id.clone(),
+    /// Return a [`FirstHop`](crate::FirstHop) object to represent this guard.
+    pub(crate) fn get_external_rep(&self) -> crate::FirstHop {
+        crate::FirstHop {
+            id: self.id.clone().into(),
             orports: self.orports.clone(),
         }
     }
+
+    /// Record that a given fallback has told us about clock skew.
+    pub(crate) fn note_skew(&mut self, observation: SkewObservation) {
+        self.clock_skew = Some(observation);
+    }
+
+    /// Return the most recent clock skew observation for this guard, if we have
+    /// made one.
+    pub(crate) fn skew(&self) -> Option<&SkewObservation> {
+        self.clock_skew.as_ref()
+    }
+
+    /// Testing only: Return true if this guard was ever contacted successfully.
+    #[cfg(test)]
+    pub(crate) fn confirmed(&self) -> bool {
+        self.confirmed_at.is_some()
+    }
 }
 
 impl tor_linkspec::ChanTarget for Guard {
@@ -578,10 +691,10 @@ impl tor_linkspec::ChanTarget for Guard {
         &self.orports[..]
     }
     fn ed_identity(&self) -> &Ed25519Identity {
-        &self.id.ed25519
+        &self.id.0.ed25519
     }
     fn rsa_identity(&self) -> &RsaIdentity {
-        &self.id.rsa
+        &self.id.0.rsa
     }
 }
 
@@ -600,42 +713,17 @@ enum GuardDisabled {
     },
 }
 
-/// Return the interval after which we should retry a guard that has
-/// been failing for the last `failing`.
+/// Return a new RetryDelay tracker for a guard.
 ///
-/// If the guard `is_primary`, we use a more aggressive retry schedule.
-fn retry_interval(is_primary: bool, failing: Duration) -> Duration {
-    /// One minute.
-    const MIN: Duration = Duration::from_secs(60);
-    /// One hour.
-    const HOUR: Duration = Duration::from_secs(60 * 60);
-    /// One (normal) day.
-    const DAY: Duration = Duration::from_secs(24 * 60 * 60);
-
-    // TODO-SPEC: This matches tor, not guardspec.
-    // TODO: Hardcoding this feels ugly.
-    #[allow(clippy::collapsible_else_if)]
-    if is_primary {
-        if failing < 6 * HOUR {
-            10 * MIN
-        } else if failing < 4 * DAY {
-            90 * MIN
-        } else if failing < 7 * DAY {
-            4 * HOUR
-        } else {
-            9 * HOUR
-        }
+/// `is_primary should be true if the guard is primary.
+fn retry_schedule(is_primary: bool) -> RetryDelay {
+    let minimum = if is_primary {
+        Duration::from_secs(30)
     } else {
-        if failing < 6 * HOUR {
-            HOUR
-        } else if failing < 4 * DAY {
-            4 * HOUR
-        } else if failing < 7 * DAY {
-            18 * HOUR
-        } else {
-            36 * HOUR
-        }
-    }
+        Duration::from_secs(150)
+    };
+
+    RetryDelay::from_duration(minimum)
 }
 
 /// The recent history of circuit activity on this guard.
@@ -719,41 +807,44 @@ mod test {
         let g = basic_guard();
 
         assert_eq!(g.guard_id(), &id);
-        assert_eq!(g.ed_identity(), &id.ed25519);
-        assert_eq!(g.rsa_identity(), &id.rsa);
+        assert_eq!(g.ed_identity(), &id.0.ed25519);
+        assert_eq!(g.rsa_identity(), &id.0.rsa);
         assert_eq!(g.addrs(), &["127.0.0.7:7777".parse().unwrap()]);
         assert_eq!(g.reachable(), Reachable::Unknown);
         assert_eq!(g.reachable(), Reachable::default());
 
         use crate::GuardUsageBuilder;
-        let usage1 = GuardUsageBuilder::new()
-            .push_restriction(GuardRestriction::AvoidId([22; 32].into()))
-            .build()
-            .unwrap();
-        let usage2 = GuardUsageBuilder::new()
-            .push_restriction(GuardRestriction::AvoidId([13; 32].into()))
-            .build()
-            .unwrap();
+        let mut usage1 = GuardUsageBuilder::new();
+        usage1
+            .restrictions()
+            .push(GuardRestriction::AvoidId([22; 32].into()));
+        let usage1 = usage1.build().unwrap();
+        let mut usage2 = GuardUsageBuilder::new();
+        usage2
+            .restrictions()
+            .push(GuardRestriction::AvoidId([13; 32].into()));
+        let usage2 = usage2.build().unwrap();
         let usage3 = GuardUsage::default();
-        let usage4 = GuardUsageBuilder::new()
-            .push_restriction(GuardRestriction::AvoidId([22; 32].into()))
-            .push_restriction(GuardRestriction::AvoidId([13; 32].into()))
-            .build()
-            .unwrap();
-        let usage5 = GuardUsageBuilder::new()
-            .push_restriction(GuardRestriction::AvoidAllIds(
-                vec![[22; 32].into(), [13; 32].into()].into_iter().collect(),
-            ))
-            .build()
-            .unwrap();
-        let usage6 = GuardUsageBuilder::new()
-            .push_restriction(GuardRestriction::AvoidAllIds(
-                vec![[99; 32].into(), [100; 32].into()]
-                    .into_iter()
-                    .collect(),
-            ))
-            .build()
-            .unwrap();
+        let mut usage4 = GuardUsageBuilder::new();
+        usage4
+            .restrictions()
+            .push(GuardRestriction::AvoidId([22; 32].into()));
+        usage4
+            .restrictions()
+            .push(GuardRestriction::AvoidId([13; 32].into()));
+        let usage4 = usage4.build().unwrap();
+        let mut usage5 = GuardUsageBuilder::new();
+        usage5.restrictions().push(GuardRestriction::AvoidAllIds(
+            vec![[22; 32].into(), [13; 32].into()].into_iter().collect(),
+        ));
+        let usage5 = usage5.build().unwrap();
+        let mut usage6 = GuardUsageBuilder::new();
+        usage6.restrictions().push(GuardRestriction::AvoidAllIds(
+            vec![[99; 32].into(), [100; 32].into()]
+                .into_iter()
+                .collect(),
+        ));
+        let usage6 = usage6.build().unwrap();
 
         assert!(g.conforms_to_usage(&usage1));
         assert!(!g.conforms_to_usage(&usage2));
@@ -790,27 +881,6 @@ mod test {
         assert!(!g3.conforms_to_usage(&dir_usage));
     }
 
-    #[test]
-    fn retry_interval_check() {
-        const MIN: Duration = Duration::from_secs(60);
-        const HOUR: Duration = Duration::from_secs(60 * 60);
-        const DAY: Duration = Duration::from_secs(24 * 60 * 60);
-
-        assert_eq!(retry_interval(true, MIN), 10 * MIN);
-        assert_eq!(retry_interval(true, 5 * MIN), 10 * MIN);
-        assert_eq!(retry_interval(true, 7 * HOUR), 90 * MIN);
-        assert_eq!(retry_interval(true, 24 * HOUR), 90 * MIN);
-        assert_eq!(retry_interval(true, 5 * DAY), 4 * HOUR);
-        assert_eq!(retry_interval(true, 100 * DAY), 9 * HOUR);
-
-        assert_eq!(retry_interval(false, MIN), HOUR);
-        assert_eq!(retry_interval(false, 5 * MIN), HOUR);
-        assert_eq!(retry_interval(false, 7 * HOUR), 4 * HOUR);
-        assert_eq!(retry_interval(false, 24 * HOUR), 4 * HOUR);
-        assert_eq!(retry_interval(false, 5 * DAY), 18 * HOUR);
-        assert_eq!(retry_interval(false, 100 * DAY), 36 * HOUR);
-    }
-
     #[test]
     fn record_attempt() {
         let t1 = Instant::now() - Duration::from_secs(10);
@@ -834,23 +904,26 @@ mod test {
         let t2 = Instant::now();
 
         let mut g = basic_guard();
-        assert!(g.failing_since.is_none());
         g.record_failure(t1, true);
-        assert_eq!(g.failing_since, Some(t1));
+        assert!(g.retry_schedule.is_some());
         assert_eq!(g.reachable(), Reachable::Unreachable);
-        assert_eq!(g.retry_at, Some(t1 + Duration::from_secs(600)));
+        let retry1 = g.retry_at.unwrap();
+        assert_eq!(retry1, t1 + Duration::from_secs(30));
 
         g.record_failure(t2, true);
-        assert_eq!(g.failing_since, Some(t1));
+        let retry2 = g.retry_at.unwrap();
+        assert!(retry2 >= t2 + Duration::from_secs(30));
+        assert!(retry2 <= t2 + Duration::from_secs(200));
     }
 
     #[test]
     fn record_success() {
         let t1 = Instant::now() - Duration::from_secs(10);
         // has to be in the future, since the guard's "added_at" time is based on now.
-        let t2 = SystemTime::now() + Duration::from_secs(300 * 86400);
+        let now = SystemTime::now();
+        let t2 = now + Duration::from_secs(300 * 86400);
         let t3 = Instant::now() + Duration::from_secs(310 * 86400);
-        let t4 = SystemTime::now() + Duration::from_secs(320 * 86400);
+        let t4 = now + Duration::from_secs(320 * 86400);
 
         let mut g = basic_guard();
         g.record_failure(t1, true);
@@ -860,7 +933,6 @@ mod test {
         assert_eq!(g.reachable(), Reachable::Reachable);
         assert_eq!(conf, NewlyConfirmed::Yes);
         assert!(g.retry_at.is_none());
-        assert!(g.failing_since.is_none());
         assert!(g.confirmed_at.unwrap() <= t2);
         assert!(g.confirmed_at.unwrap() >= t2 - Duration::from_secs(12 * 86400));
         let confirmed_at_orig = g.confirmed_at;
@@ -872,7 +944,6 @@ mod test {
         assert_eq!(conf, NewlyConfirmed::No);
         assert_eq!(g.reachable(), Reachable::Reachable);
         assert!(g.retry_at.is_none());
-        assert!(g.failing_since.is_none());
         assert_eq!(g.confirmed_at, confirmed_at_orig);
     }
 
@@ -899,7 +970,6 @@ mod test {
         g.consider_retry(g.retry_at.unwrap() + Duration::from_secs(1));
         assert!(g.retry_at.is_none());
         assert_eq!(g.reachable(), Reachable::Unknown);
-        assert_eq!(g.failing_since, Some(t1));
     }
 
     #[test]
@@ -947,7 +1017,8 @@ mod test {
         assert!(Some(guard22.added_at) <= Some(now));
 
         // Can we still get the relay back?
-        let r = guard22.id.get_relay(&netdir).unwrap();
+        let id: FirstHopId = guard22.id.clone().into();
+        let r = id.get_relay(&netdir).unwrap();
         assert_eq!(r.ed_identity(), relay22.ed_identity());
 
         // Can we check on the guard's weight?
@@ -960,7 +1031,8 @@ mod test {
             vec![],
             now,
         );
-        assert!(guard255.id.get_relay(&netdir).is_none());
+        let id: FirstHopId = guard255.id.clone().into();
+        assert!(id.get_relay(&netdir).is_none());
         assert!(guard255.get_weight(&netdir).is_none());
     }
 
@@ -1012,7 +1084,8 @@ mod test {
 
         // Try a guard that is in netdir, but not netdir2.
         let mut guard22 = Guard::new(GuardId::new([22; 32].into(), [22; 20].into()), vec![], now);
-        let relay22 = guard22.id.get_relay(&netdir).unwrap();
+        let id22: FirstHopId = guard22.id.clone().into();
+        let relay22 = id22.get_relay(&netdir).unwrap();
         assert_eq!(guard22.listed_in(&netdir), Some(true));
         guard22.update_from_netdir(&netdir);
         assert_eq!(guard22.unlisted_since, None); // It's listed.
@@ -1121,4 +1194,62 @@ mod test {
             assert_eq!(g.reachable(), *post);
         }
     }
+
+    #[test]
+    fn dir_status() {
+        // We're going to see how directory failures interact with circuit
+        // failures.
+
+        use crate::GuardUsageBuilder;
+        let mut g = basic_guard();
+        let inst = Instant::now();
+        let st = SystemTime::now();
+        let sec = Duration::from_secs(1);
+        let params = GuardParams::default();
+        let dir_usage = GuardUsageBuilder::new()
+            .kind(GuardUsageKind::OneHopDirectory)
+            .build()
+            .unwrap();
+        let data_usage = GuardUsage::default();
+
+        // Record a circuit success.
+        let _ = g.record_success(st, &params);
+        assert_eq!(g.next_retry(&dir_usage), None);
+        assert!(g.ready_for_usage(&dir_usage, inst));
+        assert_eq!(g.next_retry(&data_usage), None);
+        assert!(g.ready_for_usage(&data_usage, inst));
+
+        // Record a dircache failure.  This does not influence data usage.
+        g.record_external_failure(ExternalActivity::DirCache, inst);
+        assert_eq!(g.next_retry(&data_usage), None);
+        assert!(g.ready_for_usage(&data_usage, inst));
+        let next_dir_retry = g.next_retry(&dir_usage).unwrap();
+        assert!(next_dir_retry >= inst + GUARD_DIR_RETRY_FLOOR);
+        assert!(!g.ready_for_usage(&dir_usage, inst));
+        assert!(g.ready_for_usage(&dir_usage, next_dir_retry));
+
+        // Record a circuit success again.  This does not make the guard usable
+        // as a directory cache.
+        let _ = g.record_success(st, &params);
+        assert!(g.ready_for_usage(&data_usage, inst));
+        assert!(!g.ready_for_usage(&dir_usage, inst));
+
+        // Record a circuit failure.
+        g.record_failure(inst + sec * 10, true);
+        let next_circ_retry = g.next_retry(&data_usage).unwrap();
+        assert!(!g.ready_for_usage(&data_usage, inst + sec * 10));
+        assert!(!g.ready_for_usage(&dir_usage, inst + sec * 10));
+        assert_eq!(
+            g.next_retry(&dir_usage).unwrap(),
+            std::cmp::max(next_circ_retry, next_dir_retry)
+        );
+
+        // Record a directory success.  This won't supersede the circuit
+        // failure.
+        g.record_external_success(ExternalActivity::DirCache);
+        assert_eq!(g.next_retry(&data_usage).unwrap(), next_circ_retry);
+        assert_eq!(g.next_retry(&dir_usage).unwrap(), next_circ_retry);
+        assert!(!g.ready_for_usage(&dir_usage, inst + sec * 10));
+        assert!(!g.ready_for_usage(&data_usage, inst + sec * 10));
+    }
 }
