@@ -97,12 +97,11 @@
 #![deny(missing_docs)]
 #![warn(noop_method_call)]
 #![deny(unreachable_pub)]
-#![deny(clippy::all)]
+#![warn(clippy::all)]
 #![deny(clippy::await_holding_lock)]
 #![deny(clippy::cargo_common_metadata)]
 #![deny(clippy::cast_lossless)]
 #![deny(clippy::checked_conversions)]
-#![warn(clippy::clone_on_ref_ptr)]
 #![warn(clippy::cognitive_complexity)]
 #![deny(clippy::debug_assert_with_mut_call)]
 #![deny(clippy::exhaustive_enums)]
@@ -131,35 +130,48 @@
 //     confirmed
 //     filtered
 
+use educe::Educe;
 use futures::channel::mpsc;
-use futures::task::{SpawnError, SpawnExt};
+use futures::task::SpawnExt;
 use serde::{Deserialize, Serialize};
 use std::collections::{HashMap, HashSet};
-use std::convert::{TryFrom, TryInto};
 use std::net::SocketAddr;
 use std::sync::{Arc, Mutex};
 use std::time::{Duration, Instant, SystemTime};
+use tor_proto::ClockSkew;
 use tracing::{debug, info, trace, warn};
 
+use tor_config::{define_list_builder_accessors, define_list_builder_helper};
 use tor_llcrypto::pk;
 use tor_netdir::{params::NetParameters, NetDir, Relay};
 use tor_persist::{DynStorageHandle, StateMgr};
 use tor_rtcompat::Runtime;
 
 mod daemon;
+mod dirstatus;
+mod err;
+mod events;
+pub mod fallback;
 mod filter;
 mod guard;
+mod ids;
 mod pending;
 mod sample;
+mod skew;
 mod util;
 
+pub use err::{GuardMgrError, PickGuardError};
+pub use events::ClockSkewEvents;
 pub use filter::GuardFilter;
+pub use ids::FirstHopId;
 pub use pending::{GuardMonitor, GuardStatus, GuardUsable};
-pub use sample::PickGuardError;
+pub use skew::SkewEstimate;
 
 use pending::{PendingRequest, RequestId};
 use sample::GuardSet;
 
+use crate::ids::FirstHopIdInner;
+
 /// A "guard manager" that selects and remembers a persistent set of
 /// guard nodes.
 ///
@@ -227,8 +239,20 @@ struct GuardMgrInner {
     /// same guard.
     waiting: Vec<PendingRequest>,
 
+    /// A list of fallback directories used to access the directory system
+    /// when no other directory information is yet known.
+    // TODO: reconfigure when the configuration changes.
+    fallbacks: fallback::FallbackState,
+
     /// Location in which to store persistent state.
     storage: DynStorageHandle<GuardSets>,
+
+    /// A sender object to publish changes in our estimated clock skew.
+    send_skew: postage::watch::Sender<Option<SkewEstimate>>,
+
+    /// A receiver object to hand out to observers who want to know about
+    /// changes in our estimated clock skew.
+    recv_skew: events::ClockSkewEvents,
 }
 
 /// Persistent state for a guard manager, as serialized to disk.
@@ -259,7 +283,11 @@ impl<R: Runtime> GuardMgr<R> {
     ///
     /// It won't be able to hand out any guards until
     /// [`GuardMgr::update_network`] has been called.
-    pub fn new<S>(runtime: R, state_mgr: S) -> Result<Self, GuardMgrError>
+    pub fn new<S>(
+        runtime: R,
+        state_mgr: S,
+        fallbacks: fallback::FallbackList,
+    ) -> Result<Self, GuardMgrError>
     where
         S: StateMgr + Send + Sync + 'static,
     {
@@ -270,6 +298,10 @@ impl<R: Runtime> GuardMgr<R> {
         // try to migrate it instead, but that's beyond the stability guarantee
         // that we're getting at this stage of our (pre-0.1) development.
         let state = storage.load()?.unwrap_or_default();
+
+        let (send_skew, recv_skew) = postage::watch::channel();
+        let recv_skew = ClockSkewEvents { inner: recv_skew };
+
         let inner = Arc::new(Mutex::new(GuardMgrInner {
             guards: state,
             last_primary_retry_time: runtime.now(),
@@ -277,17 +309,24 @@ impl<R: Runtime> GuardMgr<R> {
             ctrl,
             pending: HashMap::new(),
             waiting: Vec::new(),
+            fallbacks: fallbacks.into(),
             storage,
+            send_skew,
+            recv_skew,
         }));
         {
             let weak_inner = Arc::downgrade(&inner);
             let rt_clone = runtime.clone();
-            runtime.spawn(daemon::report_status_events(rt_clone, weak_inner, rcv))?;
+            runtime
+                .spawn(daemon::report_status_events(rt_clone, weak_inner, rcv))
+                .map_err(|e| GuardMgrError::from_spawn("guard status event reporter", e))?;
         }
         {
             let rt_clone = runtime.clone();
             let weak_inner = Arc::downgrade(&inner);
-            runtime.spawn(daemon::run_periodic(rt_clone, weak_inner))?;
+            runtime
+                .spawn(daemon::run_periodic(rt_clone, weak_inner))
+                .map_err(|e| GuardMgrError::from_spawn("periodic guard updater", e))?;
         }
         Ok(GuardMgr { runtime, inner })
     }
@@ -336,6 +375,13 @@ impl<R: Runtime> GuardMgr<R> {
             == 0
     }
 
+    /// Mark every guard as potentially retriable, regardless of how recently we
+    /// failed to connect to it.
+    pub fn mark_all_guards_retriable(&self) {
+        let mut inner = self.inner.lock().expect("Poisoned lock");
+        inner.guards.active_guards_mut().mark_all_guards_retriable();
+    }
+
     /// Update the state of this [`GuardMgr`] based on a new or modified
     /// [`NetDir`] object.
     ///
@@ -352,6 +398,14 @@ impl<R: Runtime> GuardMgr<R> {
         inner.update(now, Some(netdir));
     }
 
+    /// Replace the fallback list held by this GuardMgr with `new_list`.
+    pub fn replace_fallback_list(&self, list: fallback::FallbackList) {
+        let mut fallbacks: fallback::FallbackState = list.into();
+        let mut inner = self.inner.lock().expect("Poisoned lock");
+        std::mem::swap(&mut inner.fallbacks, &mut fallbacks);
+        inner.fallbacks.take_status_from(fallbacks);
+    }
+
     /// Replace the current [`GuardFilter`] used by this `GuardMgr`.
     ///
     /// (Since there is only one kind of filter right now, there's no
@@ -401,7 +455,7 @@ impl<R: Runtime> GuardMgr<R> {
 
     /// Select a guard for a given [`GuardUsage`].
     ///
-    /// On success, we return a [`GuardId`] object to identify which
+    /// On success, we return a [`FirstHopId`] object to identify which
     /// guard we have picked, a [`GuardMonitor`] object that the
     /// caller can use to report whether its attempt to use the guard
     /// succeeded or failed, and a [`GuardUsable`] future that the
@@ -428,7 +482,7 @@ impl<R: Runtime> GuardMgr<R> {
         &self,
         usage: GuardUsage,
         netdir: Option<&NetDir>,
-    ) -> Result<(Guard, GuardMonitor, GuardUsable), PickGuardError> {
+    ) -> Result<(FirstHop, GuardMonitor, GuardUsable), PickGuardError> {
         let now = self.runtime.now();
         let wallclock = self.runtime.wallclock();
 
@@ -438,18 +492,11 @@ impl<R: Runtime> GuardMgr<R> {
         // it should _probably_ not hurt.)
         inner.guards.active_guards_mut().consider_all_retries(now);
 
-        let (origin, guard_id) = inner.select_guard_with_retries(&usage, netdir, wallclock)?;
-        let guard = inner
-            .guards
-            .active_guards()
-            .get(&guard_id)
-            .expect("Selected guard that wasn't in our sample!?")
-            .get_external_rep();
+        let (origin, guard) = inner.select_guard_with_expand(&usage, netdir, now, wallclock)?;
+        trace!(?guard, ?usage, "Guard selected");
 
-        trace!(?guard_id, ?usage, "Guard selected");
-
-        let (usable, usable_sender) = if origin.is_primary() {
-            (GuardUsable::new_primary(), None)
+        let (usable, usable_sender) = if origin.usable_immediately() {
+            (GuardUsable::new_usable_immediately(), None)
         } else {
             let (u, snd) = GuardUsable::new_uncertain();
             (u, Some(snd))
@@ -463,24 +510,92 @@ impl<R: Runtime> GuardMgr<R> {
         // don't want to acknowledge the net as down before that point, since
         // we don't mark all the primary guards as retriable unless
         // we've been forced to non-primary guards.
-        let net_has_been_down = inner
-            .guards
-            .active_guards_mut()
-            .all_primary_guards_are_unreachable()
-            && tor_proto::time_since_last_incoming_traffic() >= inner.params.internet_down_timeout;
+        let net_has_been_down =
+            if let Some(duration) = tor_proto::time_since_last_incoming_traffic() {
+                inner
+                    .guards
+                    .active_guards_mut()
+                    .all_primary_guards_are_unreachable()
+                    && duration >= inner.params.internet_down_timeout
+            } else {
+                // TODO: Is this the correct behavior in this case?
+                false
+            };
 
         let pending_request =
-            pending::PendingRequest::new(guard_id.clone(), usage, usable_sender, net_has_been_down);
+            pending::PendingRequest::new(guard.id.clone(), usage, usable_sender, net_has_been_down);
         inner.pending.insert(request_id, pending_request);
 
-        inner
-            .guards
-            .active_guards_mut()
-            .record_attempt(&guard_id, now);
+        match &guard.id.0 {
+            FirstHopIdInner::Guard(id) => inner.guards.active_guards_mut().record_attempt(id, now),
+            FirstHopIdInner::Fallback(_) => {
+                // We don't record attempts for fallbacks; we only care when
+                // they have failed.
+            }
+        }
 
         Ok((guard, monitor, usable))
     }
 
+    /// Record that _after_ we built a circuit with a guard, something described
+    /// in `external_failure` went wrong with it.
+    pub fn note_external_failure(
+        &self,
+        ed_identity: &pk::ed25519::Ed25519Identity,
+        rsa_identity: &pk::rsa::RsaIdentity,
+        external_failure: ExternalActivity,
+    ) {
+        let now = self.runtime.now();
+        let mut inner = self.inner.lock().expect("Poisoned lock");
+
+        for id in inner.lookup_ids(ed_identity, rsa_identity) {
+            match &id.0 {
+                FirstHopIdInner::Guard(id) => {
+                    inner.guards.active_guards_mut().record_failure(
+                        id,
+                        Some(external_failure),
+                        now,
+                    );
+                }
+                FirstHopIdInner::Fallback(id) => {
+                    if external_failure == ExternalActivity::DirCache {
+                        inner.fallbacks.note_failure(id, now);
+                    }
+                }
+            }
+        }
+    }
+
+    /// Record that _after_ we built a circuit with a guard, some activity
+    /// described in `external_activity` was successful with it.
+    pub fn note_external_success(
+        &self,
+        ed_identity: &pk::ed25519::Ed25519Identity,
+        rsa_identity: &pk::rsa::RsaIdentity,
+        external_activity: ExternalActivity,
+    ) {
+        let mut inner = self.inner.lock().expect("Poisoned lock");
+
+        inner.record_external_success(
+            ed_identity,
+            rsa_identity,
+            external_activity,
+            self.runtime.wallclock(),
+        );
+    }
+
+    /// Return a stream of events about our estimated clock skew; these events
+    /// are `None` when we don't have enough information to make an estimate,
+    /// and `Some(`[`SkewEstimate`]`)` otherwise.
+    ///
+    /// Note that this stream can be lossy: if the estimate changes more than
+    /// one before you read from the stream, you might only get the most recent
+    /// update.
+    pub fn skew_events(&self) -> ClockSkewEvents {
+        let inner = self.inner.lock().expect("Poisoned lock");
+        inner.recv_skew.clone()
+    }
+
     /// Ensure that the message queue is flushed before proceeding to
     /// the next step.  Used for testing.
     #[cfg(test)]
@@ -498,6 +613,15 @@ impl<R: Runtime> GuardMgr<R> {
     }
 }
 
+/// An activity that can succeed or fail, and whose success or failure can be
+/// attributed to a guard.
+#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+#[non_exhaustive]
+pub enum ExternalActivity {
+    /// The activity of using the guard as a directory cache.
+    DirCache,
+}
+
 impl GuardSets {
     /// Return a reference to the currently active set of guards.
     ///
@@ -516,8 +640,8 @@ impl GuardSets {
 
     /// Update all non-persistent state for the guards in this object with the
     /// state in `other`.
-    fn copy_status_from(&mut self, other: &GuardSets) {
-        self.default.copy_status_from(&other.default);
+    fn copy_status_from(&mut self, other: GuardSets) {
+        self.default.copy_status_from(other.default);
     }
 }
 
@@ -575,8 +699,8 @@ impl GuardMgrInner {
     /// Replace the active guard state with `new_state`, preserving
     /// non-persistent state for any guards that are retained.
     fn replace_guards_with(&mut self, mut new_guards: GuardSets, now: SystemTime) {
-        new_guards.copy_status_from(&self.guards);
-        self.guards = new_guards;
+        std::mem::swap(&mut self.guards, &mut new_guards);
+        self.guards.copy_status_from(new_guards);
         self.update(now, None);
     }
 
@@ -610,14 +734,47 @@ impl GuardMgrInner {
         &mut self,
         request_id: RequestId,
         status: GuardStatus,
+        skew: Option<ClockSkew>,
         runtime: &impl tor_rtcompat::SleepProvider,
     ) {
         if let Some(mut pending) = self.pending.remove(&request_id) {
             // If there was a pending request matching this RequestId, great!
             let guard_id = pending.guard_id();
             trace!(?guard_id, ?status, "Received report of guard status");
-            match status {
-                GuardStatus::Success => {
+
+            // First, handle the skew report (if any)
+            if let Some(skew) = skew {
+                let now = runtime.now();
+                let observation = skew::SkewObservation { skew, when: now };
+
+                match &guard_id.0 {
+                    FirstHopIdInner::Guard(id) => {
+                        self.guards.active_guards_mut().record_skew(id, observation);
+                    }
+                    FirstHopIdInner::Fallback(id) => {
+                        self.fallbacks.note_skew(id, observation);
+                    }
+                }
+                // TODO: We call this whenever we receive an observed clock
+                // skew. That's not the perfect timing for two reasons.  First
+                // off, it might be too frequent: it does an O(n) calculation,
+                // which isn't ideal.  Second, it might be too infrequent: after
+                // an hour has passed, a given observation won't be up-to-date
+                // any more, and we might want to recalculate the skew
+                // accordingly.
+                self.update_skew(now);
+            }
+
+            match (status, &guard_id.0) {
+                (GuardStatus::Failure, FirstHopIdInner::Fallback(id)) => {
+                    // We used a fallback, and we weren't able to build a circuit through it.
+                    self.fallbacks.note_failure(id, runtime.now());
+                }
+                (_, FirstHopIdInner::Fallback(_)) => {
+                    // We don't record any other kind of circuit activity if we
+                    // took the entry from the fallback list.
+                }
+                (GuardStatus::Success, FirstHopIdInner::Guard(id)) => {
                     // If we had gone too long without any net activity when we
                     // gave out this guard, and now we're seeing a circuit
                     // succeed, tell the primary guards that they might be
@@ -628,8 +785,9 @@ impl GuardMgrInner {
 
                     // The guard succeeded.  Tell the GuardSet.
                     self.guards.active_guards_mut().record_success(
-                        guard_id,
+                        id,
                         &self.params,
+                        None,
                         runtime.wallclock(),
                     );
                     // Either tell the request whether the guard is
@@ -645,22 +803,20 @@ impl GuardMgrInner {
                         self.waiting.push(pending);
                     }
                 }
-                GuardStatus::Failure => {
+                (GuardStatus::Failure, FirstHopIdInner::Guard(id)) => {
                     self.guards
                         .active_guards_mut()
-                        .record_failure(guard_id, runtime.now());
+                        .record_failure(id, None, runtime.now());
                     pending.reply(false);
                 }
-                GuardStatus::AttemptAbandoned => {
-                    self.guards
-                        .active_guards_mut()
-                        .record_attempt_abandoned(guard_id);
+                (GuardStatus::AttemptAbandoned, FirstHopIdInner::Guard(id)) => {
+                    self.guards.active_guards_mut().record_attempt_abandoned(id);
                     pending.reply(false);
                 }
-                GuardStatus::Indeterminate => {
+                (GuardStatus::Indeterminate, FirstHopIdInner::Guard(id)) => {
                     self.guards
                         .active_guards_mut()
-                        .record_indeterminate_result(guard_id);
+                        .record_indeterminate_result(id);
                     pending.reply(false);
                 }
             };
@@ -683,6 +839,53 @@ impl GuardMgrInner {
         self.expire_and_answer_pending_requests(runtime.now());
     }
 
+    /// Helper to implement `GuardMgr::note_external_success()`.
+    ///
+    /// (This has to be a separate function so that we can borrow params while
+    /// we have `mut self` borrowed.)
+    fn record_external_success(
+        &mut self,
+        ed_identity: &pk::ed25519::Ed25519Identity,
+        rsa_identity: &pk::rsa::RsaIdentity,
+        external_activity: ExternalActivity,
+        now: SystemTime,
+    ) {
+        for id in self.lookup_ids(ed_identity, rsa_identity) {
+            match &id.0 {
+                FirstHopIdInner::Guard(id) => {
+                    self.guards.active_guards_mut().record_success(
+                        id,
+                        &self.params,
+                        Some(external_activity),
+                        now,
+                    );
+                }
+                FirstHopIdInner::Fallback(id) => {
+                    if external_activity == ExternalActivity::DirCache {
+                        self.fallbacks.note_success(id);
+                    }
+                }
+            }
+        }
+    }
+
+    /// Return an iterator over all of the clock skew observations we've made
+    /// for guards or fallbacks.
+    fn skew_observations(&self) -> impl Iterator<Item = &skew::SkewObservation> {
+        self.fallbacks
+            .skew_observations()
+            .chain(self.guards.active_guards().skew_observations())
+    }
+
+    /// Recalculate our estimated clock skew, and publish it to anybody who
+    /// cares.
+    fn update_skew(&mut self, now: Instant) {
+        let estimate = skew::SkewEstimate::estimate_skew(self.skew_observations(), now);
+        // TODO: we might want to do this only conditionally, when the skew
+        // estimate changes.
+        *self.send_skew.borrow_mut() = estimate;
+    }
+
     /// If the circuit built because of a given [`PendingRequest`] may
     /// now be used (or discarded), return `Some(true)` or
     /// `Some(false)` respectively.
@@ -690,12 +893,17 @@ impl GuardMgrInner {
     /// Return None if we can't yet give an answer about whether such
     /// a circuit is usable.
     fn guard_usability_status(&self, pending: &PendingRequest, now: Instant) -> Option<bool> {
-        self.guards.active_guards().circ_usability_status(
-            pending.guard_id(),
-            pending.usage(),
-            &self.params,
-            now,
-        )
+        match &pending.guard_id().0 {
+            FirstHopIdInner::Guard(id) => self.guards.active_guards().circ_usability_status(
+                id,
+                pending.usage(),
+                &self.params,
+                now,
+            ),
+            // Fallback circuits are usable immediately, since we don't have to wait to
+            // see whether any _other_ circuit succeeds or fails.
+            FirstHopIdInner::Fallback(_) => Some(true),
+        }
     }
 
     /// For requests that have been "waiting" for an answer for too long,
@@ -745,6 +953,36 @@ impl GuardMgrInner {
         std::mem::swap(&mut waiting, &mut self.waiting);
     }
 
+    /// Return every currently extant FirstHopId for a guard or fallback
+    /// directory matching the provided keys.
+    ///
+    /// # TODO
+    ///
+    /// This function should probably not exist; it's only used so that dirmgr
+    /// can report successes or failures, since by the time it observes them it
+    /// doesn't know whether its circuit came from a guard or a fallback.  To
+    /// solve that, we'll need CircMgr to record and report which one it was
+    /// using, which will take some more plumbing.
+    fn lookup_ids(
+        &self,
+        ed_identity: &pk::ed25519::Ed25519Identity,
+        rsa_identity: &pk::rsa::RsaIdentity,
+    ) -> Vec<FirstHopId> {
+        let mut vec = Vec::with_capacity(2);
+
+        let id = ids::GuardId::new(*ed_identity, *rsa_identity);
+        if self.guards.active_guards().contains(&id) {
+            vec.push(id.into());
+        }
+
+        let id = ids::FallbackId::new(*ed_identity, *rsa_identity);
+        if self.fallbacks.contains(&id) {
+            vec.push(id.into());
+        }
+
+        vec
+    }
+
     /// Run any periodic events that update guard status, and return a
     /// duration after which periodic events should next be run.
     pub(crate) fn run_periodic_events(&mut self, wallclock: SystemTime, now: Instant) -> Duration {
@@ -753,41 +991,84 @@ impl GuardMgrInner {
         Duration::from_secs(1) // TODO: Too aggressive.
     }
 
-    /// Try to select a guard, expanding the sample or marking guards retriable
-    /// if the first attempts fail.
-    fn select_guard_with_retries(
+    /// Try to select a guard, expanding the sample if the first attempt fails.
+    fn select_guard_with_expand(
         &mut self,
         usage: &GuardUsage,
         netdir: Option<&NetDir>,
-        now: SystemTime,
-    ) -> Result<(sample::ListKind, GuardId), PickGuardError> {
+        now: Instant,
+        wallclock: SystemTime,
+    ) -> Result<(sample::ListKind, FirstHop), PickGuardError> {
         // Try to find a guard.
-        if let Ok(s) = self.guards.active_guards().pick_guard(usage, &self.params) {
-            return Ok(s);
-        }
+        let first_error = match self.select_guard_once(usage, now) {
+            Ok(res1) => return Ok(res1),
+            Err(e) => {
+                trace!("Couldn't select guard on first attempt: {}", e);
+                e
+            }
+        };
 
         // That didn't work. If we have a netdir, expand the sample and try again.
         if let Some(dir) = netdir {
             trace!("No guards available, trying to extend the sample.");
-            self.update(now, Some(dir));
+            self.update(wallclock, Some(dir));
             if self
                 .guards
                 .active_guards_mut()
-                .extend_sample_as_needed(now, &self.params, dir)
+                .extend_sample_as_needed(wallclock, &self.params, dir)
             {
                 self.guards
                     .active_guards_mut()
                     .select_primary_guards(&self.params);
-                if let Ok(s) = self.guards.active_guards().pick_guard(usage, &self.params) {
-                    return Ok(s);
+                match self.select_guard_once(usage, now) {
+                    Ok(res) => return Ok(res),
+                    Err(e) => {
+                        trace!("Couldn't select guard after expanding sample: {}", e);
+                    }
                 }
             }
         }
 
-        // That didn't work either. Mark everybody as potentially retriable.
-        info!("All guards seem down. Marking them retriable and trying again.");
-        self.guards.active_guards_mut().mark_all_guards_retriable();
-        self.guards.active_guards().pick_guard(usage, &self.params)
+        // Okay, that didn't work either.  If we were asked for a directory
+        // guard, then we may be able to use a fallback.
+        if usage.kind == GuardUsageKind::OneHopDirectory {
+            return self.select_fallback(now);
+        }
+
+        // Couldn't extend the sample or use a fallback; return the original error.
+        Err(first_error)
+    }
+
+    /// Helper: try to pick a single guard, without retrying on failure.
+    fn select_guard_once(
+        &self,
+        usage: &GuardUsage,
+        now: Instant,
+    ) -> Result<(sample::ListKind, FirstHop), PickGuardError> {
+        let (source, id) = self
+            .guards
+            .active_guards()
+            .pick_guard(usage, &self.params, now)?;
+        let guard = self
+            .guards
+            .active_guards()
+            .get(&id)
+            .expect("Selected guard that wasn't in our sample!?")
+            .get_external_rep();
+
+        Ok((source, guard))
+    }
+
+    /// Helper: Select a fallback directory.
+    ///
+    /// Called when we have no guard information to use. Return values are as
+    /// for [`GuardMgr::select_guard()`]
+    fn select_fallback(
+        &self,
+        now: Instant,
+    ) -> Result<(sample::ListKind, FirstHop), PickGuardError> {
+        let fallback = self.fallbacks.choose(&mut rand::thread_rng(), now)?;
+        Ok((sample::ListKind::Fallback, fallback.clone()))
     }
 }
 
@@ -883,48 +1164,18 @@ impl TryFrom<&NetParameters> for GuardParams {
     }
 }
 
-/// A unique cryptographic identifier for a selected guard.
-///
-/// (This is implemented internally using both of the guard's Ed25519
-/// and RSA identities.)
-#[derive(Clone, Debug, Serialize, Deserialize, Eq, PartialEq, Hash)]
-pub struct GuardId {
-    /// Ed25519 identity key for a a guard
-    ed25519: pk::ed25519::Ed25519Identity,
-    /// RSA identity fingerprint for a a guard
-    rsa: pk::rsa::RsaIdentity,
-}
-
-impl GuardId {
-    /// Return a new, manually constructed GuardId
-    fn new(ed25519: pk::ed25519::Ed25519Identity, rsa: pk::rsa::RsaIdentity) -> Self {
-        Self { ed25519, rsa }
-    }
-
-    /// Extract a GuardId from a Relay object.
-    pub(crate) fn from_relay(relay: &tor_netdir::Relay<'_>) -> Self {
-        Self::new(*relay.id(), *relay.rsa_id())
-    }
-
-    /// Return the relay in `netdir` that corresponds to this ID, if there
-    /// is one.
-    pub fn get_relay<'a>(&self, netdir: &'a NetDir) -> Option<Relay<'a>> {
-        netdir.by_id_pair(&self.ed25519, &self.rsa)
-    }
-}
-
-/// Representation of a guard, as returned by [`GuardMgr::select_guard()`].
+/// Representation of a guard or fallback, as returned by [`GuardMgr::select_guard()`].
 #[derive(Debug, Clone, Eq, PartialEq)]
-pub struct Guard {
+pub struct FirstHop {
     /// The guard's identities
-    id: GuardId,
+    id: FirstHopId,
     /// The addresses at which the guard can be contacted.
     orports: Vec<SocketAddr>,
 }
 
-impl Guard {
+impl FirstHop {
     /// Return the identities of this guard.
-    pub fn id(&self) -> &GuardId {
+    pub fn id(&self) -> &FirstHopId {
         &self.id
     }
     /// Look up this guard in `netdir`.
@@ -934,27 +1185,29 @@ impl Guard {
 }
 
 // This is somewhat redundant with the implementation in crate::guard::Guard.
-impl tor_linkspec::ChanTarget for Guard {
+impl tor_linkspec::ChanTarget for FirstHop {
     fn addrs(&self) -> &[SocketAddr] {
         &self.orports[..]
     }
     fn ed_identity(&self) -> &pk::ed25519::Ed25519Identity {
-        &self.id.ed25519
+        &self.id.as_ref().ed25519
     }
     fn rsa_identity(&self) -> &pk::rsa::RsaIdentity {
-        &self.id.rsa
+        &self.id.as_ref().rsa
     }
 }
 
 /// The purpose for which we plan to use a guard.
 ///
 /// This can affect the guard selection algorithm.
-#[derive(Clone, Debug, Eq, PartialEq)]
+#[derive(Clone, Debug, Eq, PartialEq, Educe)]
+#[educe(Default)]
 #[non_exhaustive]
 pub enum GuardUsageKind {
     /// We want to use this guard for a data circuit.
     ///
     /// (This encompasses everything except the `OneHopDirectory` case.)
+    #[educe(Default)]
     Data,
     /// We want to use this guard for a one-hop, non-anonymous
     /// directory request.
@@ -964,12 +1217,6 @@ pub enum GuardUsageKind {
     OneHopDirectory,
 }
 
-impl Default for GuardUsageKind {
-    fn default() -> GuardUsageKind {
-        GuardUsageKind::Data
-    }
-}
-
 /// A set of parameters describing how a single guard should be selected.
 ///
 /// Used as an argument to [`GuardMgr::select_guard`].
@@ -980,8 +1227,28 @@ pub struct GuardUsage {
     #[builder(default)]
     kind: GuardUsageKind,
     /// A list of restrictions on which guard may be used.
-    #[builder(default)]
-    restrictions: Vec<GuardRestriction>,
+    ///
+    /// The default is the empty list.
+    #[builder(sub_builder, setter(custom))]
+    restrictions: GuardRestrictionList,
+}
+
+/// List of socket restricteionesses, as configured
+pub type GuardRestrictionList = Vec<GuardRestriction>;
+
+define_list_builder_helper! {
+    pub struct GuardRestrictionListBuilder {
+        restrictions: [GuardRestriction],
+    }
+    built: GuardRestrictionList = restrictions;
+    default = vec![];
+    item_build: |restriction| Ok(restriction.clone());
+}
+
+define_list_builder_accessors! {
+    struct GuardUsageBuilder {
+        pub restrictions: [GuardRestriction],
+    }
 }
 
 impl GuardUsageBuilder {
@@ -989,14 +1256,6 @@ impl GuardUsageBuilder {
     pub fn new() -> Self {
         Self::default()
     }
-
-    /// Add `restriction` to the list of restrictions on this guard usage.
-    pub fn push_restriction(&mut self, restriction: GuardRestriction) -> &mut Self {
-        self.restrictions
-            .get_or_insert_with(Vec::new)
-            .push(restriction);
-        self
-    }
 }
 
 /// A restriction that applies to a single request for a guard.
@@ -1006,7 +1265,7 @@ impl GuardUsageBuilder {
 /// They're suitable for things like making sure that we don't start
 /// and end a circuit at the same relay, or requiring a specific
 /// subprotocol version for certain kinds of requests.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
 #[non_exhaustive]
 pub enum GuardRestriction {
     /// Don't pick a guard with the provided Ed25519 identity.
@@ -1015,24 +1274,6 @@ pub enum GuardRestriction {
     AvoidAllIds(HashSet<pk::ed25519::Ed25519Identity>),
 }
 
-/// An error caused while creating or updating a guard manager.
-#[derive(Clone, Debug, thiserror::Error)]
-#[non_exhaustive]
-pub enum GuardMgrError {
-    /// An error derived from the state manager.
-    #[error("Problem accessing persistent state")]
-    State(#[from] tor_persist::Error),
-    /// An error that occurred while trying to spawn a daemon task.
-    #[error("Unable to spawn task")]
-    Spawn(#[source] Arc<SpawnError>),
-}
-
-impl From<SpawnError> for GuardMgrError {
-    fn from(e: SpawnError) -> GuardMgrError {
-        GuardMgrError::Spawn(Arc::new(e))
-    }
-}
-
 #[cfg(test)]
 mod test {
     #![allow(clippy::unwrap_used)]
@@ -1052,7 +1293,7 @@ mod test {
         let statemgr = TestingStateMgr::new();
         let have_lock = statemgr.try_lock().unwrap();
         assert!(have_lock.held());
-        let guardmgr = GuardMgr::new(rt, statemgr.clone()).unwrap();
+        let guardmgr = GuardMgr::new(rt, statemgr.clone(), [].into()).unwrap();
         let (con, mds) = testnet::construct_network().unwrap();
         let override_p = "guard-min-filtered-sample-size=5 guard-n-primary-guards=2"
             .parse()
@@ -1089,7 +1330,7 @@ mod test {
             drop(guardmgr);
 
             // Try reloading from the state...
-            let guardmgr2 = GuardMgr::new(rt.clone(), statemgr.clone()).unwrap();
+            let guardmgr2 = GuardMgr::new(rt.clone(), statemgr.clone(), [].into()).unwrap();
             guardmgr2.update_network(&netdir);
 
             // Since the guard was confirmed, we should get the same one this time!
@@ -1155,7 +1396,64 @@ mod test {
 
             let (guard, _mon, _usable) = guardmgr.select_guard(u, Some(&netdir)).unwrap();
             // Make sure that the filter worked.
-            assert_eq!(guard.id().rsa.as_bytes()[0] % 4, 0);
+            assert_eq!(guard.id().as_ref().rsa.as_bytes()[0] % 4, 0);
+        });
+    }
+
+    #[test]
+    fn external_status() {
+        use tor_linkspec::ChanTarget;
+        test_with_all_runtimes!(|rt| async move {
+            let (guardmgr, _statemgr, netdir) = init(rt);
+            let data_usage = GuardUsage::default();
+            let dir_usage = GuardUsageBuilder::new()
+                .kind(GuardUsageKind::OneHopDirectory)
+                .build()
+                .unwrap();
+            guardmgr.update_network(&netdir);
+            {
+                // Override this parameter, so that we can get deterministic results below.
+                let mut inner = guardmgr.inner.lock().unwrap();
+                inner.params.dir_parallelism = 1;
+            }
+
+            let (guard, mon, _usable) = guardmgr
+                .select_guard(data_usage.clone(), Some(&netdir))
+                .unwrap();
+            mon.succeeded();
+
+            // Record that this guard gave us a bad directory object.
+            guardmgr.note_external_failure(
+                guard.ed_identity(),
+                guard.rsa_identity(),
+                ExternalActivity::DirCache,
+            );
+
+            // We ask for another guard, for data usage.  We should get the same
+            // one as last time, since the director failure doesn't mean this
+            // guard is useless as a primary guard.
+            let (g2, mon, _usable) = guardmgr.select_guard(data_usage, Some(&netdir)).unwrap();
+            assert_eq!(g2.ed_identity(), guard.ed_identity());
+            mon.succeeded();
+
+            // But if we ask for a guard for directory usage, we should get a
+            // different one, since the last guard we gave out failed.
+            let (g3, mon, _usable) = guardmgr
+                .select_guard(dir_usage.clone(), Some(&netdir))
+                .unwrap();
+            assert_ne!(g3.ed_identity(), guard.ed_identity());
+            mon.succeeded();
+
+            // Now record a success for for directory usage.
+            guardmgr.note_external_success(
+                guard.ed_identity(),
+                guard.rsa_identity(),
+                ExternalActivity::DirCache,
+            );
+
+            // Now that the guard is working as a cache, asking for it should get us the same guard.
+            let (g4, _mon, _usable) = guardmgr.select_guard(dir_usage, Some(&netdir)).unwrap();
+            assert_eq!(g4.ed_identity(), guard.ed_identity());
         });
     }
 }
